@@ -22,44 +22,12 @@ using namespace pqxx;
 // called template1, or "host=foo.bar.net user=smith" to connect to a
 // backend running on host foo.bar.net, logging in as user smith.
 
-#ifdef _WIN32
-#include <windows.h>
-#else
-extern "C"
-{
-#ifdef PQXX_HAVE_SYS_SELECT_H
-#include <sys/select.h>
-#else
-#include <sys/time.h>
-#include <sys/types.h>
-#include <unistd.h>
-#endif
-}
-#endif	// _WIN32
-
 namespace
 {
 
 int Backend_PID = 0;
 
 
-#ifndef _WIN32
-// Reasonably portable way to sleep for a given number of milliseconds
-/* This definition is not needed on Windows, which provides its own Sleep()
- * function with the same semantics.
- */
-void Sleep(int ms)
-{
-  fd_set F;
-  FD_ZERO(&F);
-  struct timeval timeout;
-  timeout.tv_sec = ms / 1000;
-  timeout.tv_usec = (ms % 1000) * 1000;
-  if (select(0, &F, &F, &F, &timeout) == -1)
-    throw runtime_error(strerror(errno));
-}
-#endif	// _WIN32
-
 // Sample implementation of trigger handler
 class TestTrig : public trigger
 {
@@ -132,7 +100,10 @@ int main(int, char *argv[])
       if (notifs)
 	throw logic_error("Got " + to_string(notifs) + 
 	    " unexpected notification(s)!");
-      Sleep(500);
+      // Sleep one second using a libpqxx-internal function.  Kids, don't try
+      // this at home!  The pqxx::internal namespace is not for third-party use
+      // and may change radically at any time.
+      pqxx::internal::sleep_seconds(1);
       notifs = C.get_notifs();
       cout << ".";
     }
